clc; clear all; close all;

%% 1. definition of the cost function handle
cost_function_hdl = @phase_estimation_cost_function;

%% 2. definition of the set of evaluation points

    % For this problem, the points to be evaluated are the phase values 
    % of the sinusoid. 
    
    % We first define a vector of K phase values...
    K = 100;
    phi = linspace(0,2*pi,K)';
    % ... and we arrange them in the form how they are estimated, i.e. 
    % into a [K x M] matrix, where M is the number of unknowns
    ev_points = [cos(phi) -sin(phi)]; % [K x M] matrix
    
%% 3. definition of the observations

    % Here we compute the noise-free observations, i.e. the samples of a
    % noise-free sinusoid ad frequency 'omega' with phase extracted from
    % the vector 'phases':  
    
    N = 6;        % number of observations
    omega = pi/4; % frequency
    
    observations = zeros(N,K);
    for n = 1:N
        for k = 1:K
            observations(n,k) = cos(omega*(n-1)+phi(k));
        end
    end
    
    
%% 4. definition of params

    % For this problem, the only parameters 
    % of the cost function is the frequency:
    params = omega;
    
    
%% 5 .definition of the estimation problem

    prob = EPT_problem(cost_function_hdl, ev_points, observations, params);


%% 6. definition of the observation covariance

    % We assume the noise to be Gaussian and with independent components on
    % each observation, so that the covariance matrix is diagonal
    var_noise = 0.001; % noise variance
    Pc = var_noise * eye(N); % [N x N] matrix 


%% 7. prediction of the covariance matrix

    Px = EPT_cov(prob,Pc); % Px is a [M x M x K] matrix
                           % containing the covariance of
                           % the estimation for each
                           % of the K test points

% End of using EPT. 
% -------------------------------------------------------------------------

%% ------------------------------------------------------------------------
% Now some operations on the EPT results:

    % Estimate the variance of the phase estimation error from Px (see the
    % EPT guide for details). 
    % Extract the standard deviation of measurement error from Px
    
    EPT_std_phi = zeros(K,1);
    for k = 1:K
        x1 =  cos(phi(k));
        x2 = -sin(phi(k));
        T = [x2/(x1^2 + x2^2) -x1/(x1^2 + x2^2)]; % transformation matrix   
        EPT_std_phi(k) = sqrt(T*Px(:,:,k)*T'); % compute standard deviation
    end
    
%% ------------------------------------------------------------------------
% We now compare the theoretical results with one obtained from
% Montecarlo simulations:
    
    
    % Simulation parameters
    %----------------------------------------------------------------------
    
    n_it = 1000; % number of realizations
    
    c = zeros(N,K); % recompute noise-free observations
    for k = 1:K
        c(:,k) = cos(omega*(0:1:N-1)'+phi(k));
    end

    % matrix H (see the EPT guide for details)
    H = [cos(omega*(0:1:N-1))', sin(omega*(0:1:N-1))'];
    pinvH = inv(H'*H)*H';
    
    % Start the simulations
    %----------------------------------------------------------------------    
    
    SIM_std_phi = zeros(K,1); % initialize a vector containing the standard
                              % deviation of the results
                             
    for k = 1:K   % for each test point 
        
        % initialize a vector for the estimation error
        err_phi = zeros(n_it,1);
        
        for i = 1:n_it % for each realization
            
            % add noise on the observation
            c_it = c(:,k) + sqrt(var_noise)*randn(N,1);
            
            % estimate the phase
            x = (pinvH*c_it)';
            est_phi = atan2(-x(2),x(1));

            % compute phase error            
            temp_err = phi(k) - mod(est_phi,2*pi);
            if temp_err>=0
                if temp_err<=pi
                    err_phi(i) = -temp_err;
                else
                    err_phi(i) = 2*pi - temp_err;
                end
            else
                if abs(temp_err)<=pi
                    err_phi(i) = -temp_err;
                else
                    err_phi(i) = -(2*pi+temp_err);
                end                    
            end   

        end

        % compute the standard deviation for the k-th test point
        SIM_std_phi(k) = std(err_phi);
        
    end

    
    % Plot the results
    %---------------------------------------------------------------------- 
    figure(1)
    plot(phi*180/pi,EPT_std_phi*180/pi,'k');
    hold on
    plot(phi*180/pi,SIM_std_phi*180/pi,'b');
    axis tight
    legend('Theoretical','Simulated');
    xlabel('\phi [deg]','interpreter','tex','fontsize',16);
    ylabel('\sigma_\phi [deg]','interpreter','tex','fontsize',16);
    set(gca,'Xtick',[0 90 180 270 360]);
    set(gca,'Fontname','Times new roman','Fontsize',18);
    title('Standard deviation of estimation error');