clc; clear all; close all;

% Error propagation for reflector localization using TOAs.

%% 1. definition of the cost function handle

    cost_function_hdl = @ref_loc_cost_function;


%% 2. definition of the set of evaluation points

    % We define a grid of test reflector lines defined by their 
    % distance 'rho' and orientation 'alpha' with respect to the origin
    %----------------------------------------------------------------------

    num_o = 9;  % number of orientations
    num_d = 10;  % number of distances
    K = num_o*num_d; % total number of test reflector positions
                                      
    or_ax  = linspace(0,2*pi,num_o);      % the orientation ranges... 
                                          % ...from 0 to 2*pi
    
    min_d = 1;                             
    max_d = 4;  
    dist_ax = linspace(min_d,max_d,num_d); % the distance ranges... 
                                           % ...from 'min_d' to 'max_d'
    
    % grid definition                                   
    [grid_o,grid_d] = meshgrid(or_ax,dist_ax);    
    pts = [grid_o(:) grid_d(:)];
    
    % conversion of the reflector lines in cartesian form, i.e. in the
    % parameter vector [l_1 l_2 l_3] defining the line of equation
    % l_1x + l_2y + l_3 = 0. 
    % Notice that we force l_3 = 1, since we no test reflector lines pass
    % trough the origin.
    
    ref_line = zeros(K,3);
    for k=1:K
        rho = pts(k,2);
        alpha = pts(k,1);
        ref_line(k,:) = [cos(alpha) sin(alpha) -rho];
        ref_line(k,:) = ref_line(k,:)/ref_line(k,3); % force l3 = 1
    end
    
    % We now define the evaluation points. The variables to be estimated
    % are l_1 and l_2, thus M = 2
    
    ev_points = ref_line(:,1:2); % a [K x 2] matrix


%% 3. definition of the observations

    % The theoretical observations are the euclidean distances between the
    % microphones and the mirror source generated by the real one
    % ---------------------------------------------------------------------
    
    % source position
    src_pos = [0 0]; % the source is in (0,0);
    
    % microphone positions (N sensors on a circle):
    N = 5; % number of microphones
    angs = linspace(0,2*pi,N+1)';
    angs = angs(1:end-1);
    r = 0.3;
    mic_pos = r*[cos(angs) sin(angs)]; % mic. positions
        
    % calculate the noise-free reflective distances
    observations = zeros(N,K);
    
    for k = 1:K
        
        % mirror source position
        l1 = ev_points(k,1);
        l2 = ev_points(k,2);

        xs = src_pos(1);
        ys = src_pos(2);

        xr = -(l1^2*xs+2*l1+2*l1*l2*ys-l2^2*xs)/(l1^2 + l2^2);
        yr = (l1^2*ys-2*l1*l2*xs-2*l2-l2^2*ys)/(l1^2 + l2^2);

        mir_src = [xr yr];  
        
        % length of the reflective paths
        for n = 1:N                              
            observations(n,k) = norm(mir_src-mic_pos(n,:));      
        end
    end


%% 4. definition of params

    % embed the microphone positions in params:
    params.mic_pos = mic_pos; 


%% 5 .definition of the estimation problem

    prob = EPT_problem(cost_function_hdl, ev_points, observations, params);


%% 6. definition of the observation covariance

    sig = 0.01;        % standard deviation of distance error
    Pc = sig^2*eye(N); % diagonal matrix  

    
%% 7. prediction of the covariance matrix

    Px = EPT_cov(prob,Pc); 
                       

 % End of using EPT. 
% -------------------------------------------------------------------------

%% ------------------------------------------------------------------------
% Now some operations on the EPT results:

    % Convert the results from cartesian paramaters 
    % to polar (distance/orientation) parameters
    
    Pp = zeros(2,2,K);      % covariance matrices of the estimated
                            % polar parameters
                            
    P_std_dev = zeros(2,K); % standard deviation of the
                            % estimated polar parameters
            
    % Compute covariance and standard deviation
    for k = 1:K
        
        l1 = ev_points(k,1);
        l2 = ev_points(k,2);
        l3 = 1;
        
        nr2 = l1^2 + l2^2;
        T = [-l1/(nr2^(3/2)), -l2/(nr2^(3/2));...
             -l2/nr2        , l1/nr2         ];
                                
        Pp(:,:,k) = T*Px(:,:,k)*T';       % covariance matrix
        
        P_std_dev(1,k) = sqrt(Pp(1,1,k)); % standard deviation
        P_std_dev(2,k) = sqrt(Pp(2,2,k));
        
    end

    % Plot results (standard deviation)
    std_o = reshape(P_std_dev(1,:),num_o,num_d);
    std_d = reshape(P_std_dev(2,:),num_o,num_d);
    
    hfig = figure;
    set(hfig,'Units', 'Normalized', 'OuterPosition', [0 0 1 1]); 
    
    subplot(1,2,1);
    colormap('gray');
    imagesc([min_d max_d],[0 360],std_o*180/pi);colorbar;
    xlabel('Distance [m]','interpreter','tex','fontsize',15);
    ylabel('Orientation [deg]','interpreter','tex','fontsize',15);
    set(gca,'xtick',[1 2 3 4]);
    set(gca,'ytick',[0 90 180 270 360]);    
    set(gca,'fontname','times new roman','fontsize',15)
    title('Std. dev. of orientation error [deg]','interpreter','tex','fontsize',15);
    axis xy; axis square
        
    subplot(1,2,2);
    colormap('gray');
    imagesc([min_d max_d],[0 360],std_d);colorbar;
    xlabel('Distance [m]','interpreter','tex','fontsize',15);
    ylabel('Orientation [deg]','interpreter','tex','fontsize',15);
    set(gca,'xtick',[1 2 3 4]);
    set(gca,'ytick',[0 90 180 270 360]);    
    set(gca,'fontname','times new roman','fontsize',15)
    title('Std. dev. of distance error [m]','interpreter','tex','fontsize',15);
    axis xy; axis square        
    