clc; clear all; close all;

%% 1. definition of the cost function handle

    cost_function = @SRDLS_cost_function;

    
%% 2. definition of the set of evaluation points

    % For this problem, the evaluation points are the possible source 
    % locations we want to test. At this purpose we choose to define a 2D
    % grid of sources.
    
    % We first define the grid parameters. 
    l_x = 3.9; % x grid length in [m]
    l_y = 3.9; % y grid length in [m]
    num_x = 10; % number of sources on x
    num_y = 10; % number of sources on y

    % We then define the grid saving each source position in the matrix
    % ev_points = [xs_1 ys_1;
    %               ...  ...;
    %              xs_K ys_K];
    delta_x = l_x / (num_x-1);
    delta_y = l_y / (num_y-1);
    num_ev = num_x*num_y;
    ev_points = zeros(num_ev,2);
    counter = 1;
    for i = 1:num_x
        for j = 1:num_y
            ev_points(counter,:) = [delta_x*(i-1) - l_x/2, -(-delta_y*(j-1) + l_y/2)];
            counter = counter + 1;
        end
    end

    
%% 3. definition of the observations

    % Here we compute the noise-free observations, i.e. the range
    % differences of arrival between pairs of microphones for each source
    % position.
    
    % We define the microphones positions
    mic_pos = [ 2  2; ...
               -2  2; ...
               -2 -2; ...
                2 -2];
            
    % We compute the microphone-source distances 
    distance = zeros(size(mic_pos,1),size(ev_points,1));
    for n = 1:size(mic_pos,1),
        for k = 1:size(ev_points,1),
            distance(n,k) = norm(mic_pos(n,:)-ev_points(k,:));
        end
    end
    
    % We compute the range differences
    observations = zeros(size(mic_pos,1),size(ev_points,1));
    for n = 1:size(mic_pos,1)
        for k = 1:size(ev_points,1)
            observations(n,k) = -(distance(1,k)-distance(n,k));
        end
    end
    observations = observations(2:end,:);

    
%% 4. definition of params

    % For this problem, the only parameters we use are the microphones
    % positions
    params = mic_pos;

    
%% 5 .definition of the estimation problem

    prob = EPT_problem(cost_function, ev_points, observations, params);


%% 6. definition of the observation covariance

    % We assume the noise to be Gaussian and with independent components on
    % each observation, so that the covariance matrix is diagonal
    var_noise = 0.03^2; % noise variance
    Pc = var_noise * eye(size(observations,1)); % [N x N] matrix 

    
%% 7. prediction of the covariance matrix

    Px = EPT_cov(prob,Pc); % Px is a [M x M x K] matrix
                           % containing the covariance of
                           % the estimation for each
                           % of the K test points


%% ------------------------------------------------------------------------
% Now some operations on the EPT results:

    % Extract the RMSE of measurement error from Px 
    RMSE = zeros(size(Px,3),size(Px,1));
    for m = 1:size(Px,1)
        for k = 1:size(Px,3)
            RMSE(k,m) = sqrt(Px(m,m,k));
        end
    end

    % Plot the RMSE on x coordinate for every source position
    ax_lim = [min(prob.ev_points(:,1)) max(prob.ev_points(:,1)) ...
              min(prob.ev_points(:,2)) max(prob.ev_points(:,2))];
    xRMSE = reshape(RMSE(:,1),num_y,num_x);
    
    figure(),
    imagesc(ax_lim(1:2),ax_lim(3:4),xRMSE);
    colormap('gray'), colorbar
    set(gca,'YDir','normal')
    set(gca,'Fontname','Times new roman','Fontsize',18);
    title('RMSE on x [m]')
    xlabel('x [m]','interpreter','tex','fontsize',16)
    ylabel('y [m]','interpreter','tex','fontsize',16)
